"""
pyrheem - Rheem EcoNet API Client

A Python library and CLI for controlling Rheem water heaters via the EcoNet API.

Library Usage:
    from pyrheem import RheemEcoNetAPI, WaterHeater, Location

    api = RheemEcoNetAPI("email@example.com", "password")
    if api.login() and api.get_all_data():
        # Get first water heater
        heater = list(api.water_heaters.values())[0]
        print(f"Current temp: {heater.setpoint}F")

        # Set temperature
        api.set_temperature(heater, 120)

        # Cleanup
        api.disconnect()

CLI Usage:
    rheem --list                    # List all devices
    rheem --status                  # Get status as JSON
    rheem --temp 120                # Set temperature
    rheem --mode energy             # Set to energy saver mode
    rheem                           # Interactive mode
"""

__version__ = "0.1.3"
__author__ = "Riley Porter"

from .api import RheemEcoNetAPI
from .models import Location, RheemSession, WaterHeater

__all__ = [
    "RheemEcoNetAPI",
    "RheemSession",
    "Location",
    "WaterHeater",
    "__version__",
]



"""Data models for Rheem EcoNet API"""

from dataclasses import dataclass, field
from typing import Any, Dict, List


@dataclass
class RheemSession:
    """Session information for authenticated user"""

    user_token: str = ""
    user_id: str = ""
    account_id: str = ""
    cloud_url: str = ""
    cloud_port: int = 1884


@dataclass
class Location:
    """A location (home) containing devices"""

    location_id: str
    name: str
    address: str
    devices: List["WaterHeater"] = field(default_factory=list)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "location_id": self.location_id,
            "name": self.name,
            "address": self.address,
            "devices": [d.to_dict() for d in self.devices],
        }


@dataclass
class WaterHeater:
    """A Rheem water heater device"""

    serial_number: str
    device_name: str
    display_name: str
    device_type: str
    location_id: str
    location_name: str
    setpoint: int = 120
    setpoint_min: int = 95
    setpoint_max: int = 140
    mode: str = ""
    mode_value: int = 0
    running: str = ""
    connected: bool = False
    enabled: bool = True
    raw_data: Dict[str, Any] = field(default_factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "serial_number": self.serial_number,
            "device_name": self.device_name,
            "display_name": self.display_name,
            "location": self.location_name,
            "setpoint": self.setpoint,
            "setpoint_range": [self.setpoint_min, self.setpoint_max],
            "mode": self.mode,
            "running": self.running,
            "connected": self.connected,
            "enabled": self.enabled,
        }



"""Command-line interface for pyrheem"""

import argparse
import json
import os
import sys
from pathlib import Path
from typing import Optional

# Load .env file
try:
    from dotenv import load_dotenv

    load_dotenv(Path.cwd() / ".env")
    load_dotenv()
except ImportError:
    pass

from .api import RheemEcoNetAPI
from .models import WaterHeater


def print_device_list(api: RheemEcoNetAPI):
    """Print formatted list of locations and devices"""
    locations = api.get_locations_list()

    if not locations:
        print("No locations found")
        return

    print("\nLocations and Devices:")
    print("=" * 60)

    for loc_idx, location in enumerate(locations):
        print(f"\n[Location {loc_idx}] {location.name}")
        print(f"  ID: {location.location_id}")
        if location.address:
            print(f"  Address: {location.address}")

        if not location.devices:
            print("  (No water heaters)")
            continue

        for dev_idx, device in enumerate(location.devices):
            status = "Connected" if device.connected else "Disconnected"
            print(f"\n  [Device {dev_idx}] {device.display_name}")
            print(f"    Serial: {device.serial_number}")
            print(f"    Setpoint: {device.setpoint}F (range: {device.setpoint_min}-{device.setpoint_max})")
            print(f"    Mode: {device.mode}")
            print(f"    Status: {device.running} | {status}")

    print("\n" + "=" * 60)
    print("Usage examples:")
    print("  rheem --location 0 --device 0 --temp 120")
    print('  rheem --location "My Home" --device "Electric" --temp 115')
    print("=" * 60)


def interactive_mode(api: RheemEcoNetAPI):
    """Interactive command mode"""
    print("\n" + "=" * 60)
    print("Interactive Mode - Commands:")
    print("  list                          - List locations and devices")
    print("  use <loc_idx> <dev_idx>       - Select device for commands")
    print("  temp <temperature>            - Set temperature")
    print("  mode <energy|performance>     - Set mode")
    print("  enable / disable              - Enable/disable device")
    print("  status                        - Show current device status")
    print("  refresh                       - Refresh data from server")
    print("  quit                          - Exit")
    print("=" * 60 + "\n")

    current_device: Optional[WaterHeater] = None

    # Default to first device
    locations = api.get_locations_list()
    if locations and locations[0].devices:
        current_device = locations[0].devices[0]
        print(f"Selected: {current_device.display_name} @ {current_device.location_name}")

    while True:
        try:
            prompt = f"rheem [{current_device.display_name if current_device else 'no device'}]> "
            cmd = input(prompt).strip()
            if not cmd:
                continue

            parts = cmd.split()
            action = parts[0].lower()

            if action in ("quit", "exit", "q"):
                break

            elif action == "list":
                print_device_list(api)

            elif action == "use" and len(parts) >= 3:
                loc = api.get_location(parts[1])
                if loc:
                    dev = api.get_device(loc, parts[2])
                    if dev:
                        current_device = dev
                        print(f"Selected: {dev.display_name} @ {dev.location_name}")
                    else:
                        print(f"Device not found: {parts[2]}")
                else:
                    print(f"Location not found: {parts[1]}")

            elif action == "temp" and len(parts) >= 2:
                if current_device:
                    api.set_temperature(current_device, int(parts[1]))
                else:
                    print("No device selected. Use 'use <loc> <dev>' first.")

            elif action == "mode" and len(parts) >= 2:
                if current_device:
                    api.set_mode(current_device, parts[1])
                else:
                    print("No device selected.")

            elif action == "enable":
                if current_device:
                    api.set_enabled(current_device, True)
                else:
                    print("No device selected.")

            elif action == "disable":
                if current_device:
                    api.set_enabled(current_device, False)
                else:
                    print("No device selected.")

            elif action == "status":
                if current_device:
                    api.get_all_data()
                    if current_device.serial_number in api.water_heaters:
                        current_device = api.water_heaters[current_device.serial_number]
                    print(f"\n{current_device.display_name} @ {current_device.location_name}")
                    print(f"  Setpoint: {current_device.setpoint}F")
                    print(f"  Mode: {current_device.mode}")
                    print(f"  Status: {current_device.running}")
                    print(f"  Connected: {current_device.connected}")
                else:
                    print("No device selected.")

            elif action == "refresh":
                api.get_all_data()
                print("Data refreshed.")

            else:
                print(f"Unknown command: {action}")

        except KeyboardInterrupt:
            break
        except Exception as e:
            print(f"Error: {e}")

    api.disconnect()


def main():
    """Main CLI entry point"""
    parser = argparse.ArgumentParser(
        prog="rheem",
        description="Rheem EcoNet Water Heater Control",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  rheem --list                              # List locations and devices
  rheem --status                            # Get all devices as JSON
  rheem --temp 120                          # Set temp (first device)
  rheem --location 0 --device 0 --temp 125  # Specify location/device by index
  rheem --location "Home" --device "Electric" --temp 120  # By name
  rheem --mode energy                       # Set Energy Saver mode
  rheem --mode performance                  # Set Performance mode
  rheem                                     # Interactive mode

Environment variables (or use .env file):
  RHEEM_EMAIL     - Your Rheem account email
  RHEEM_PASSWORD  - Your Rheem account password
        """,
    )

    parser.add_argument("--email", default=os.environ.get("RHEEM_EMAIL"), help="Rheem account email")
    parser.add_argument("--password", default=os.environ.get("RHEEM_PASSWORD"), help="Rheem account password")
    parser.add_argument("--location", "-L", default="0", help="Location index or name (default: 0)")
    parser.add_argument("--device", "-d", default="0", help="Device index, serial, or name (default: 0)")
    parser.add_argument("--list", "-l", action="store_true", help="List all locations and devices")
    parser.add_argument("--status", "-s", action="store_true", help="Get all device status as JSON")
    parser.add_argument("--temp", "-t", type=int, help="Set temperature in Fahrenheit")
    parser.add_argument("--mode", "-m", choices=["energy", "performance"], help="Set mode: energy or performance")
    parser.add_argument("--enable", action="store_true", help="Enable the water heater")
    parser.add_argument("--disable", action="store_true", help="Disable the water heater")
    parser.add_argument("--quiet", "-q", action="store_true", help="Quiet mode (JSON output only)")
    parser.add_argument("--interactive", "-i", action="store_true", help="Force interactive mode")

    args = parser.parse_args()

    # Check credentials
    if not args.email or not args.password:
        print("Error: Credentials required. Set RHEEM_EMAIL and RHEEM_PASSWORD")
        print("       environment variables, create a .env file, or use --email/--password")
        sys.exit(1)

    has_command = args.list or args.status or args.temp or args.mode or args.enable or args.disable
    quiet = args.quiet or (has_command and not args.interactive)

    api = RheemEcoNetAPI(args.email, args.password, quiet=quiet)

    # Login
    if not api.login():
        print('{"error": "Login failed"}' if quiet else "[-] Login failed")
        sys.exit(1)

    # Get device data
    if not api.get_all_data():
        print('{"error": "Failed to get devices"}' if quiet else "[-] Failed to get devices")
        sys.exit(1)

    if not api.water_heaters:
        print('{"error": "No water heaters found"}' if quiet else "[-] No water heaters found")
        sys.exit(1)

    # Handle commands
    if args.list:
        print_device_list(api)

    elif args.status:
        output = {"locations": [loc.to_dict() for loc in api.get_locations_list()]}
        print(json.dumps(output, indent=2))

    elif args.temp or args.mode or args.enable or args.disable:
        location = api.get_location(args.location)
        if not location:
            msg = f"Location not found: {args.location}"
            print(f'{{"error": "{msg}"}}' if quiet else f"[-] {msg}")
            sys.exit(1)

        device = api.get_device(location, args.device)
        if not device:
            msg = f"Device not found: {args.device}"
            print(f'{{"error": "{msg}"}}' if quiet else f"[-] {msg}")
            sys.exit(1)

        if not quiet:
            print(f"[*] Target: {device.display_name} @ {location.name}")

        success = False
        result = {}

        if args.temp:
            success = api.set_temperature(device, args.temp)
            result = {"success": success, "temperature": args.temp}
        elif args.mode:
            success = api.set_mode(device, args.mode)
            result = {"success": success, "mode": args.mode}
        elif args.enable:
            success = api.set_enabled(device, True)
            result = {"success": success, "enabled": True}
        elif args.disable:
            success = api.set_enabled(device, False)
            result = {"success": success, "enabled": False}

        if quiet:
            print(json.dumps(result))

        sys.exit(0 if success else 1)

    else:
        # Interactive mode
        if not quiet:
            print("\n" + "=" * 60)
            print("Rheem EcoNet API Client")
            print("=" * 60)
            print_device_list(api)

        api.get_provisioning_config()

        if api.connect_mqtt():
            interactive_mode(api)
        else:
            print("[-] Could not connect to MQTT for real-time control")

    api.disconnect()


if __name__ == "__main__":
    main()



"""Rheem EcoNet API Client"""

import json
import ssl
import time
from typing import Callable, Dict, List, Optional

import requests

from .models import Location, RheemSession, WaterHeater

# Try to import paho-mqtt
try:
    import paho.mqtt.client as mqtt

    try:
        from paho.mqtt.enums import CallbackAPIVersion

        PAHO_V2 = True
    except ImportError:
        PAHO_V2 = False
    MQTT_AVAILABLE = True
except ImportError:
    mqtt = None
    PAHO_V2 = False
    MQTT_AVAILABLE = False

# API Constants (from decompiled APK)
PROD_URL = "https://rheem.rheemconnect.com/api/v/1/"
SYSTEM_KEY = "e2e699cb0bb0bbb88fc8858cb5a401"
SYSTEM_SECRET = "E2E699CB0BE6C6FADDB1B0BC9A20"

# MQTT Topics
USER_DESIRED_TOPIC = "user/{account_id}/device/desired"
USER_REPORTED_TOPIC = "user/{account_id}/device/reported"


class RheemEcoNetAPI:
    """
    Rheem EcoNet API Client for controlling water heaters.

    Example usage as a library:
        from pyrheem import RheemEcoNetAPI

        api = RheemEcoNetAPI("email@example.com", "password")
        if api.login() and api.get_all_data():
            # Get first water heater
            heater = list(api.water_heaters.values())[0]
            print(f"Current temp: {heater.setpoint}")

            # Set temperature
            api.set_temperature(heater, 120)
    """

    def __init__(self, email: str, password: str, quiet: bool = True):
        """
        Initialize the API client.

        Args:
            email: Rheem account email
            password: Rheem account password
            quiet: If True, suppress log output (default for library use)
        """
        self.email = email
        self.password = password
        self.quiet = quiet
        self.session = RheemSession()
        self.mqtt_client = None
        self.locations: Dict[str, Location] = {}
        self.water_heaters: Dict[str, WaterHeater] = {}
        self._mqtt_connected = False
        self._on_message_callback: Optional[Callable] = None

    def _log(self, msg: str):
        if not self.quiet:
            print(msg)

    def _get_headers(self, include_auth: bool = True) -> Dict[str, str]:
        headers = {
            "Content-Type": "application/json",
            "User-Agent": "EcoNet/6.14.0 Android",
            "ClearBlade-SystemKey": SYSTEM_KEY,
            "ClearBlade-SystemSecret": SYSTEM_SECRET,
        }
        if include_auth and self.session.user_token:
            headers["ClearBlade-UserToken"] = self.session.user_token
        return headers

    def login(self) -> bool:
        """
        Authenticate with the Rheem API.

        Returns:
            True if login successful, False otherwise
        """
        url = f"{PROD_URL}user/auth"
        payload = {"email": self.email, "password": self.password}
        self._log(f"[*] Logging in as {self.email}...")

        try:
            response = requests.post(url, json=payload, headers=self._get_headers(include_auth=False))
            if response.status_code != 200:
                self._log(f"[-] Login failed: {response.status_code}")
                return False

            data = response.json()
            if "user_token" in data:
                self.session.user_token = data.get("user_token", "")
                self.session.user_id = data.get("user_id", "")
                self.session.account_id = data.get("options", {}).get("account_id", "")
                self._log("[+] Login successful!")
                return True
            return False
        except Exception as e:
            self._log(f"[-] Login error: {e}")
            return False

    def get_provisioning_config(self) -> bool:
        """Get MQTT broker configuration."""
        url = f"{PROD_URL}code/{SYSTEM_KEY}/provisioningConfig"
        self._log("[*] Getting MQTT config...")

        try:
            response = requests.post(url, headers=self._get_headers())
            data = response.json()
            if data.get("success"):
                results = data.get("results", {})
                domain = results.get("default_domain", results.get("defaultDomain", {}))
                self.session.cloud_url = domain.get("cloud_url", domain.get("cloudUrl", "rheemprod.rheemcert.com"))
                self.session.cloud_port = domain.get("cloud_port", domain.get("cloudPort", 1884))
                self._log(f"[+] MQTT: {self.session.cloud_url}:{self.session.cloud_port}")
                return True
        except Exception as e:
            self._log(f"[-] Config error: {e}")

        self.session.cloud_url = "rheemprod.rheemcert.com"
        self.session.cloud_port = 1884
        return True

    def get_all_data(self) -> bool:
        """
        Fetch all locations and devices.

        Returns:
            True if successful, False otherwise
        """
        url = f"{PROD_URL}code/{SYSTEM_KEY}/getUserDataForApp"
        self._log("[*] Fetching devices...")

        try:
            response = requests.post(url, json={"location_only": False}, headers=self._get_headers())
            if response.status_code != 200:
                return False

            data = response.json()
            self._parse_devices(data)
            return True
        except Exception as e:
            self._log(f"[-] Error: {e}")
            return False

    def _parse_devices(self, data: Dict):
        self.locations.clear()
        self.water_heaters.clear()

        locations_data = data.get("results", {}).get("locations", [])

        for loc_data in locations_data:
            location_id = loc_data.get("location_id", "")
            location_name = loc_data.get("@LOCATION_NAME", loc_data.get("name", "Unknown"))
            location_info = loc_data.get("@LOCATION_INFO", "")

            location = Location(location_id=location_id, name=location_name, address=location_info, devices=[])

            equipment_list = loc_data.get("equipment", loc_data.get("equiptments", []))

            for eq in equipment_list:
                device_type = eq.get("@TYPE", eq.get("device_type", ""))
                if "water" not in device_type.lower() and device_type != "WH":
                    continue

                serial = eq.get("serial_number", "")
                device_name = eq.get("device_name", "")
                display_name = eq.get("@NAME", {})
                display_name = (
                    display_name.get("value", "Water Heater")
                    if isinstance(display_name, dict)
                    else str(display_name) or "Water Heater"
                )

                setpoint_data = eq.get("@SETPOINT", {})
                if isinstance(setpoint_data, dict):
                    setpoint = int(setpoint_data.get("value", 120))
                    constraints = setpoint_data.get("constraints", {})
                    setpoint_min = constraints.get("lowerLimit", 95)
                    setpoint_max = constraints.get("upperLimit", 140)
                else:
                    setpoint = int(setpoint_data) if setpoint_data else 120
                    setpoint_min, setpoint_max = 95, 140

                mode_data = eq.get("@MODE", {})
                if isinstance(mode_data, dict):
                    mode = mode_data.get("status", "Unknown")
                    mode_value = int(mode_data.get("value", 0))
                else:
                    mode, mode_value = str(mode_data), 0

                enabled_data = eq.get("@ENABLED", {})
                enabled = enabled_data.get("value", 1) == 1 if isinstance(enabled_data, dict) else bool(enabled_data)

                wh = WaterHeater(
                    serial_number=serial,
                    device_name=device_name,
                    display_name=display_name,
                    device_type=device_type,
                    location_id=location_id,
                    location_name=location_name,
                    setpoint=setpoint,
                    setpoint_min=setpoint_min,
                    setpoint_max=setpoint_max,
                    mode=mode,
                    mode_value=mode_value,
                    running=eq.get("@RUNNING", ""),
                    connected=eq.get("@CONNECTED", False),
                    enabled=enabled,
                    raw_data=eq,
                )

                location.devices.append(wh)
                self.water_heaters[serial] = wh

            self.locations[location_id] = location

    def get_locations_list(self) -> List[Location]:
        """Get locations as ordered list."""
        return list(self.locations.values())

    def get_location(self, identifier: str) -> Optional[Location]:
        """
        Get location by index, ID, or name (partial match).

        Args:
            identifier: Location index (e.g., "0"), ID, or name

        Returns:
            Location if found, None otherwise
        """
        locations = self.get_locations_list()

        if identifier.isdigit():
            idx = int(identifier)
            if 0 <= idx < len(locations):
                return locations[idx]

        if identifier in self.locations:
            return self.locations[identifier]

        identifier_lower = identifier.lower()
        for loc in locations:
            if identifier_lower in loc.name.lower():
                return loc

        return None

    def get_device(self, location: Optional[Location], device_id: str) -> Optional[WaterHeater]:
        """
        Get device by index, serial, or name within a location.

        Args:
            location: Location to search in (None for all devices)
            device_id: Device index, serial number, or name

        Returns:
            WaterHeater if found, None otherwise
        """
        if location:
            devices = location.devices
        else:
            devices = list(self.water_heaters.values())

        if not devices:
            return None

        if device_id.isdigit():
            idx = int(device_id)
            if 0 <= idx < len(devices):
                return devices[idx]

        if device_id in self.water_heaters:
            return self.water_heaters[device_id]

        device_id_lower = device_id.lower()
        for dev in devices:
            if device_id_lower in dev.display_name.lower() or device_id_lower in dev.serial_number.lower():
                return dev

        return None

    # =========================================================================
    # MQTT Functions
    # =========================================================================

    @property
    def mqtt_available(self) -> bool:
        """Check if MQTT is available."""
        return MQTT_AVAILABLE

    @property
    def mqtt_connected(self) -> bool:
        """Check if MQTT is connected."""
        return self._mqtt_connected

    def connect_mqtt(self) -> bool:
        """
        Connect to MQTT broker for real-time control.

        Returns:
            True if connected successfully
        """
        if not MQTT_AVAILABLE:
            self._log("[-] paho-mqtt not installed. Run: pip install paho-mqtt")
            return False

        if not self.session.cloud_url:
            self.get_provisioning_config()

        self._log("[*] Connecting to MQTT...")
        client_id = f"python_econet_{int(time.time())}"

        try:
            if PAHO_V2:
                self.mqtt_client = mqtt.Client(
                    callback_api_version=mqtt.CallbackAPIVersion.VERSION2, client_id=client_id, protocol=mqtt.MQTTv311
                )
            else:
                self.mqtt_client = mqtt.Client(client_id=client_id, protocol=mqtt.MQTTv311)

            self.mqtt_client.username_pw_set(self.session.user_token, SYSTEM_KEY)
            self.mqtt_client.tls_set(cert_reqs=ssl.CERT_NONE)
            self.mqtt_client.tls_insecure_set(True)

            self.mqtt_client.on_connect = self._on_connect
            self.mqtt_client.on_message = self._on_message
            self.mqtt_client.on_disconnect = self._on_disconnect

            self.mqtt_client.connect(self.session.cloud_url, self.session.cloud_port, keepalive=60)
            self.mqtt_client.loop_start()

            for _ in range(10):
                if self._mqtt_connected:
                    return True
                time.sleep(0.5)

            self._log("[-] MQTT connection timeout")
            return False
        except Exception as e:
            self._log(f"[-] MQTT error: {e}")
            return False

    def set_message_callback(self, callback: Callable[[Dict], None]):
        """
        Set a callback for incoming MQTT messages.

        Args:
            callback: Function that receives parsed message dict
        """
        self._on_message_callback = callback

    def _on_connect(self, client, userdata, flags, reason_code, properties=None):
        is_success = (reason_code == 0) if isinstance(reason_code, int) else not reason_code.is_failure
        if is_success:
            self._mqtt_connected = True
            self._log("[+] MQTT connected!")
            topic = USER_REPORTED_TOPIC.format(account_id=self.session.account_id)
            client.subscribe(topic, qos=0)
        else:
            self._log(f"[-] MQTT connect failed: {reason_code}")

    def _on_message(self, client, userdata, msg):
        try:
            payload = json.loads(msg.payload.decode())
            self._log(f"\n[MQTT] {json.dumps(payload, indent=2)[:800]}")
            if self._on_message_callback:
                self._on_message_callback(payload)
        except Exception:
            pass

    def _on_disconnect(self, client, userdata, disconnect_flags_or_rc, reason_code=None, properties=None):
        self._mqtt_connected = False

    def _send_mqtt_command(self, wh: WaterHeater, **kwargs) -> bool:
        if not self._mqtt_connected:
            if not self.connect_mqtt():
                return False

        transaction_id = f"PYTHON_{int(time.time() * 1000)}"
        message = {
            "transactionId": transaction_id,
            "device_name": wh.device_name,
            "serial_number": wh.serial_number,
            **kwargs,
        }

        topic = USER_DESIRED_TOPIC.format(account_id=self.session.account_id)
        self._log(f"[*] Sending: {json.dumps(message)}")

        try:
            result = self.mqtt_client.publish(topic, json.dumps(message), qos=0)
            result.wait_for_publish(timeout=5)
            if result.rc == 0:
                self._log("[+] Command sent!")
                time.sleep(2)
                return True
        except Exception as e:
            self._log(f"[-] Error: {e}")
        return False

    def set_temperature(self, wh: WaterHeater, temperature: int) -> bool:
        """
        Set water heater temperature.

        Args:
            wh: WaterHeater device
            temperature: Target temperature in Fahrenheit

        Returns:
            True if command sent successfully
        """
        if temperature < wh.setpoint_min or temperature > wh.setpoint_max:
            self._log(f"[-] Temperature must be {wh.setpoint_min}-{wh.setpoint_max}F")
            return False
        self._log(f"[*] Setting {wh.display_name} to {temperature}F...")
        return self._send_mqtt_command(wh, **{"@SETPOINT": temperature})

    def set_mode(self, wh: WaterHeater, mode: str) -> bool:
        """
        Set water heater mode.

        Args:
            wh: WaterHeater device
            mode: "energy" or "performance"

        Returns:
            True if command sent successfully
        """
        mode_lower = mode.lower()
        if "energy" in mode_lower or "saver" in mode_lower:
            mode_value = 0
        elif "performance" in mode_lower or "high" in mode_lower:
            mode_value = 1
        else:
            self._log("[-] Invalid mode. Use: energy, performance")
            return False
        self._log(f"[*] Setting mode to {mode}...")
        return self._send_mqtt_command(wh, **{"@MODE": mode_value})

    def set_enabled(self, wh: WaterHeater, enabled: bool) -> bool:
        """
        Enable or disable water heater.

        Args:
            wh: WaterHeater device
            enabled: True to enable, False to disable

        Returns:
            True if command sent successfully
        """
        action = "Enabling" if enabled else "Disabling"
        self._log(f"[*] {action} {wh.display_name}...")
        return self._send_mqtt_command(wh, **{"@ENABLED": 1 if enabled else 0})

    def disconnect(self):
        """Disconnect from MQTT broker."""
        if self.mqtt_client:
            self.mqtt_client.loop_stop()
            self.mqtt_client.disconnect()
            self._mqtt_connected = False
